"""
Main module.

:Copyright: 2018-present Reece Dunham.
:License: MIT, see LICENSE for more details.
"""

from . import util as utils

# Info variables:
name = "area4"
__author__ = "RDIL"
author_email = "me@rdil.rocks"
description = "Dividers in Python, the easy way!"


def divider(number):
    """
    Get the divider you requested.

    :param number: The divider number (can't be 0).
    :type number: int
    :return: The requested divider
    :rtype: str
    :raises ValueError: If you request an invalid divider.
    :Example:
    :code:`area4.divider(3)` will return :code:`............`
    """
    if number == 0 or type(number) != int:
        raise ValueError('Please use a number bigger then 0!')
    else:
        try:
            return utils.get_raw_file()[number].replace("\n", "")
        except IndexError:
            raise ValueError('That divider doesn\'t exist!')


def splitter(div, *args):
    """
    Split text with dividers easily.

    :return: The newly made value.
    :rtype: str
    :param div: The divider.
    :type div: str
    """
    retstr = ""
    if type(div) is int:
        div = utils.get_raw_file()[div]
    if len(args) == 1:
        return args[0]
    for s in args:
        retstr += s
        retstr += "\n"
        retstr += div
        retstr += "\n"
    return retstr


def area4info():
    """
    Get some info about the package.

    :return: Package info.
    :rtype: str
    """
    return "Name: {0}\nAuthor: {1}\nAuthor Email: {2}\nDescription: {3}".format(
        name,
        __author__,
        author_email,
        description
    )


def make_div(unit, length=24,
             start='', end='',
             literal_unit=False):
    """
    Generate a custom divider.

    :param unit: A repeating unit.
    :type unit: str
    :param length: The maximum length (won't be exceeded) (default: 24).
    :type length: int
    :param start: Starting string.
    :type start: Optional[str]
    :param end: Ending string.
    :type end: Optional[str]
    :param literal_unit:
        If :code:`True`, it will not try to break
        unit down into smaller repeating subunits.
        Defaults to :code:`False`.
    :type literal_unit: Optional[bool]
    :return: A new, custom divider.
    :rtype: str

    :Example:
    :code:`custom_div = make_div(unit='=-', length=40, start='<', end='=>')`

    .. note::
        The generated string will be terminated
        at the specified length regardless
        of if all the input strings have been fully replicated.
        A unit > 1 length may
        not be able to be replicated to extend to the full length.
        In this situation, the
        string will be shorter than the specified length.
        Example: unit of 10 characters and a specified length of
        25 will contain 2 units for
        a total length of 20 characters.
    """
    # Reduce the size if possible to extend closer to full length:
    if not literal_unit:
        unit = utils.reduce_to_unit(unit)
    repeats = (length - len(start + end)) // len(unit)
    return (start + unit * repeats + end)[0:length]



"""
Utilities module.

:Copyright: 2018-present Reece Dunham.
:License: MIT, see LICENSE for more details.
"""

import os
import random
from functools import lru_cache


@lru_cache(maxsize=None)
def get_raw_file():
    """
    Get the raw divider file in a string array.

    Note: This function is decorated with
    :code:`functools.lru_cache`, so calling
    it a second time should be faster!

    :return: The array.
    :rtype: list
    """
    with open("{0}/dividers.txt".format(
        os.path.abspath(os.path.dirname(__file__))
    ), mode="r") as file_handler:
        lines = file_handler.readlines()
        stringbuilder = ""
        i = 0
        while i < 6:
            stringbuilder += "<>"
            i += 1
        # no longer needed
        del i
        # we need to manually inject this, GitHub thinks its
        # a conflict marker
        lines[32] = stringbuilder
        lines[35] = str(
            random.randint(0, 999999999999)
        )
        return lines


def reduce_to_unit(divider):
    """
    Reduce a repeating divider to the smallest repeating unit possible.

    This function is used by :code:`make-div`.

    :param divider: The divider.
    :type divider: str
    :return: Smallest repeating unit possible.
    :rtype: str

    :Example: 'XxXxXxX' -> 'Xx'
    """
    for unit_size in range(1, len(divider) // 2 + 1):
        length = len(divider)
        unit = divider[:unit_size]

        # Ignores mismatches in final characters:
        divider_item = divider[:unit_size * (length // unit_size)]
        if unit * (length // unit_size) == divider_item:
            return unit
    return divider  # Return original if no smaller unit


def get_divider_character(divider_id):
    """
    Get the character the divider is made of.

    :param divider_id: The divider's number.
    :type divider_id: int
    :return: The character.
    :rtype: str
    :raises ValueError: If you request an invalid divider.
    :Example:
        Get what divider 7 is made of:
        :code:`get_divider_character(7)`
        will return '='.
    """
    blacklisted = [18, 19, 22, 33, 34, 35, 222, 223, 224, 226, 233, 234, 242]
    if divider_id in blacklisted:
        return None
    try:
        return get_raw_file()[divider_id][0]
    except IndexError:
        raise ValueError("That divider doesn't exist!")


def reddit_horizontal():
    """
    Get Reddit horizontal divider.

    :return: The divider.
    :rtype: str
    """
    return "*****"


def markdown_horizontal():
    """
    Get Markdown horizontal divider.

    :return: The divider.
    :rtype: str
    """
    return "---"



"""Builds the package."""

import setuptools

# Package version:
VERSION = "2.11.1"

# Read the long description:
with open("README.md", mode="r") as FILE_HANDLER:
    LONG_DESCRIPTION = FILE_HANDLER.read()

# Keywords:
TAGS = [
    "area4",
    "dividers",
    "python",
    "utilities",
    "enhancements",
    "text"
]

# Classifiers:
CLASSIFIERS = [
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "License :: OSI Approved :: MIT License",
    "Operating System :: Microsoft",
    "Operating System :: Microsoft :: Windows :: Windows 10",
    "Operating System :: Microsoft :: Windows :: Windows 8",
    "Operating System :: Microsoft :: Windows :: Windows 8.1",
    "Operating System :: Microsoft :: Windows :: Windows 7",
    "Operating System :: MacOS",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Other OS",
    "Intended Audience :: Developers",
    "Intended Audience :: End Users/Desktop",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities",
    "Topic :: System",
    "Topic :: Terminals",
    "Topic :: Text Processing",
    "Development Status :: 5 - Production/Stable",
    "Framework :: IDLE",
    "Natural Language :: English",
]

# GitHub URL:
MainURL = "https://github.com/area4lib/area4"

# Other Project URLs:
URLs = \
    {
        "Bug Tracker": "https://github.com/area4lib/area4/issues",
        "Documentation": "https://area4.readthedocs.io/en/stable/",
        "Source Code": "https://github.com/area4lib/area4",
        "License": "https://github.com/area4lib/area4/blob/master/LICENSE",
    }


setuptools.setup(
    name="area4",
    version=VERSION,
    author="area4 Team",
    author_email="me@rdil.rocks",
    description="Dividers in Python, the easy way!",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url=MainURL,
    license="MIT",
    packages=setuptools.find_packages(),
    classifiers=CLASSIFIERS,
    project_urls=URLs,
    keywords=TAGS,
    include_package_data=True,
    zip_safe=False
)
